@@ -196,8 +196,11 @@ class DSLdapObject(DSLogging, DSLint):
         """
         e = _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=attrlist,
                                         serverctrls=self._server_controls, clientctrls=self._client_controls,
-                                        escapehatch='i am sure')[0]
-        return e.__repr__()
+                                        escapehatch='i am sure')
+        if len(e) > 0:
+            return e[0].__repr__()
+        else:
+            return ""
 
     def display_attr(self, attr):
         """Get all values of given attribute - 'attr: value'
@@ -310,6 +313,29 @@ class DSLdapObject(DSLogging, DSLint):
 
         self.set(key, value, action=ldap.MOD_ADD)
 
+    def add_many(self, *args):
+        """Add many key, value pairs in a single operation.
+        This is useful for configuration changes that require
+        atomic operation, and ease of use.
+
+        An example of usage is add_many((key, value), (key, value))
+
+        No wrapping list is needed for the arguments.
+
+        :param *args: tuples of key,value to replace.
+        :type *args: (str, str)
+        """
+
+        mods = []
+        for arg in args:
+            if isinstance(arg[1], list) or isinstance(arg[1], tuple):
+                value = ensure_list_bytes(arg[1])
+            else:
+                value = [ensure_bytes(arg[1])]
+            mods.append((ldap.MOD_ADD, ensure_str(arg[0]), value))
+        return _modify_ext_s(self._instance,self._dn, mods, serverctrls=self._server_controls,
+                                            clientctrls=self._client_controls, escapehatch='i am sure')
+
     # Basically what it means;
     def replace(self, key, value):
         """Replace an attribute with a value
@@ -623,12 +649,15 @@ class DSLdapObject(DSLogging, DSLint):
             # retrieving real(*) and operational attributes(+)
             attrs_entry = _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter,
                                                       attrlist=["*", "+"], serverctrls=self._server_controls,
-                                                      clientctrls=self._client_controls, escapehatch='i am sure')[0]
-            # getting dict from 'entry' object
-            attrs_dict = attrs_entry.data
-            # Should we normalise the attr names here to lower()?
-            # This could have unforseen consequences ...
-            return attrs_dict
+                                                      clientctrls=self._client_controls, escapehatch='i am sure')
+            if len(attrs_entry) > 0:
+                # getting dict from 'entry' object
+                attrs_dict = attrs_entry[0].data
+                # Should we normalise the attr names here to lower()?
+                # This could have unforseen consequences ...
+                return attrs_dict
+            else:
+                return {}
 
     def get_all_attrs_utf8(self, use_json=False):
         """Get a dictionary having all the attributes of the entry
@@ -643,12 +672,15 @@ class DSLdapObject(DSLogging, DSLint):
             # retrieving real(*) and operational attributes(+)
             attrs_entry = _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter,
                                                       attrlist=["*", "+"], serverctrls=self._server_controls,
-                                                      clientctrls=self._client_controls, escapehatch='i am sure')[0]
-            # getting dict from 'entry' object
-            r = {}
-            for (k, vo) in attrs_entry.data.items():
-                r[k] = ensure_list_str(vo)
-            return r
+                                                      clientctrls=self._client_controls, escapehatch='i am sure')
+            if len(attrs_entry) > 0:
+                # getting dict from 'entry' object
+                r = {}
+                for (k, vo) in attrs_entry[0].data.items():
+                    r[k] = ensure_list_str(vo)
+                return r
+            else:
+                return {}
 
     def get_attrs_vals(self, keys, use_json=False):
         self._log.debug("%s get_attrs_vals(%r)" % (self._dn, keys))
@@ -657,8 +689,11 @@ class DSLdapObject(DSLogging, DSLint):
         else:
             entry = _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter,
                                                 attrlist=keys, serverctrls=self._server_controls,
-                                                clientctrls=self._client_controls, escapehatch='i am sure')[0]
-            return entry.getValuesSet(keys)
+                                                clientctrls=self._client_controls, escapehatch='i am sure')
+            if len(entry) > 0:
+                return entry[0].getValuesSet(keys)
+            else:
+                return []
 
     def get_attrs_vals_utf8(self, keys, use_json=False):
         self._log.debug("%s get_attrs_vals_utf8(%r)" % (self._dn, keys))
@@ -666,12 +701,15 @@ class DSLdapObject(DSLogging, DSLint):
             raise ValueError("Invalid state. Cannot get properties on instance that is not ONLINE")
         entry = _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=keys,
                                             serverctrls=self._server_controls, clientctrls=self._client_controls,
-                                            escapehatch='i am sure')[0]
-        vset = entry.getValuesSet(keys)
-        r = {}
-        for (k, vo) in vset.items():
-            r[k] = ensure_list_str(vo)
-        return r
+                                            escapehatch='i am sure')
+        if len(entry) > 0:
+            vset = entry[0].getValuesSet(keys)
+            r = {}
+            for (k, vo) in vset.items():
+                r[k] = ensure_list_str(vo)
+            return r
+        else:
+            return {}
 
     def get_attr_vals(self, key, use_json=False):
         self._log.debug("%s get_attr_vals(%r)" % (self._dn, key))
@@ -685,15 +723,21 @@ class DSLdapObject(DSLogging, DSLint):
             # We have to do this in this method, because else we ignore the scope base.
             entry = _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter,
                                                 attrlist=[key], serverctrls=self._server_controls,
-                                                clientctrls=self._client_controls, escapehatch='i am sure')[0]
-            vals = entry.getValues(key)
-            if use_json:
-                result = {key: []}
-                for val in vals:
-                    result[key].append(val)
-                return result
+                                                clientctrls=self._client_controls, escapehatch='i am sure')
+            if len(entry) > 0:
+                vals = entry[0].getValues(key)
+                if use_json:
+                    result = {key: []}
+                    for val in vals:
+                        result[key].append(val)
+                    return result
+                else:
+                    return vals
             else:
-                return vals
+                if use_json:
+                    return {}
+                else:
+                    return []
 
     def get_attr_val(self, key, use_json=False):
         self._log.debug("%s getVal(%r)" % (self._dn, key))
@@ -705,8 +749,11 @@ class DSLdapObject(DSLogging, DSLint):
         else:
             entry = _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter,
                                                 attrlist=[key], serverctrls=self._server_controls,
-                                                clientctrls=self._client_controls, escapehatch='i am sure')[0]
-            return entry.getValue(key)
+                                                clientctrls=self._client_controls, escapehatch='i am sure')
+            if len(entry) > 0:
+                return entry[0].getValue(key)
+            else:
+                return ""
 
     def get_attr_val_bytes(self, key, use_json=False):
         """Get a single attribute value from the entry in bytes type
@@ -724,7 +771,7 @@ class DSLdapObject(DSLogging, DSLint):
 
         :param key: An attribute name
         :type key: str
-        :returns: A single bytes value
+        :returns: A list of bytes values
         :raises: ValueError - if instance is offline
         """
 
@@ -735,7 +782,7 @@ class DSLdapObject(DSLogging, DSLint):
 
         :param key: An attribute name
         :type key: str
-        :returns: A single bytes value
+        :returns: A single UTF8 value
         :raises: ValueError - if instance is offline
         """
 
@@ -746,7 +793,7 @@ class DSLdapObject(DSLogging, DSLint):
 
         :param key: An attribute name
         :type key: str
-        :returns: A single bytes value
+        :returns: A single lowered UTF8 value
         :raises: ValueError - if instance is offline
         """
 
@@ -761,7 +808,7 @@ class DSLdapObject(DSLogging, DSLint):
 
         :param key: An attribute name
         :type key: str
-        :returns: A single bytes value
+        :returns: A list of UTF8 values
         :raises: ValueError - if instance is offline
         """
 
@@ -772,7 +819,7 @@ class DSLdapObject(DSLogging, DSLint):
 
         :param key: An attribute name
         :type key: str
-        :returns: A single bytes value
+        :returns: A list of lowered UTF8 values
         :raises: ValueError - if instance is offline
         """
 
@@ -783,7 +830,7 @@ class DSLdapObject(DSLogging, DSLint):
 
         :param key: An attribute name
         :type key: str
-        :returns: A single bytes value
+        :returns: A single int value
         :raises: ValueError - if instance is offline
         """
 
@@ -794,7 +841,7 @@ class DSLdapObject(DSLogging, DSLint):
 
         :param key: An attribute name
         :type key: str
-        :returns: A single bytes value
+        :returns: A list of int values
         :raises: ValueError - if instance is offline
         """
 
@@ -1462,9 +1509,17 @@ class CompositeDSLdapObject(DSLdapObject):
         idx = self._find_idx(key)[-1]
         return self._entries[idx].get_attr_vals(key, use_json)
 
+    def get_attr_vals_utf8(self, key, use_json=False):
+        idx = self._find_idx(key)[-1]
+        return self._entries[idx].get_attr_vals_utf8(key, use_json)
+
     def get_attr_val(self, key, use_json=False):
         idx = self._find_idx(key)[-1]
-        return self._entries[idx].get_attr_vals(key, use_json)
+        return self._entries[idx].get_attr_val(key, use_json)
+
+    def get_attr_val_utf8(self, key, use_json=False):
+        idx = self._find_idx(key)[-1]
+        return self._entries[idx].get_attr_val_utf8(key, use_json)
 
     def add_values(self, values):
         raise DeprecationWarning("Not implemented any more.")
